import os
import os.path

import click

from .. import config


@click.group(context_settings={'help_option_names': ('-h', '--help', '-?')})
def patterncfg() -> None:
    pass


@patterncfg.command(name='list', help='list all patternutils template files')
def list() -> None:
    for d in os.listdir(config.CONFIG_PATH):
        click.secho(d, fg='blue')
        for f in os.listdir(os.path.join(config.CONFIG_PATH, d)):
            click.echo('└── {}'.format(f))


@patterncfg.command(name='add', help='add new template file')
@click.option('-t', '--file-type', default='regex', type=click.Choice(['regex', 'python', 'jinja2']),
              help='select the file type to use')
@click.option('-c', '--command', default=None, type=click.Choice(['pmatch', 'tmv', 'targs']),
              help='select the command from which to view the file')
@click.argument('name')
def add(
    file_type: str,
    command: str,
    name: str,
) -> None:
    if command == 'pmatch':
        fileext = 'regex'
    else:
        if file_type not in ('python', 'jinja2'):
            fileext = 'python'
        else:
            fileext = file_type

    click.edit(filename=os.path.join(config.CONFIG_PATH, command, '.'.join((name, fileext))))


@patterncfg.command(name='remove', help='remove template file')
@click.option('-c', '--command', default=None, type=click.Choice(['pmatch', 'tmv', 'targs']),
              help='select the command from which to view the file')
@click.argument('name')
def remove(command: str, name: str) -> None:
    files = config.find_templates(name, command)
    try:
        os.remove(files[0])
    except IndexError:
        click.secho('Cannot remove {}. It doesn\'t exist!'.format(name), fg='red', err=True)
        raise SystemExit


@patterncfg.command(name='view', help='view template file')
@click.option('-c', '--command', default=None, type=click.Choice(['pmatch', 'tmv', 'targs']),
              help='select the command from which to view the file')
@click.argument('name')
def view(name: str, command: str) -> None:

    files = config.find_templates(name, command)

    if not files:
        click.secho('No files found!', fg='red', err=True)
        raise SystemExit
    elif len(files) != 1:
        click.secho('Multiple files found!', fg='red', err=True)
        for f in files:
            click.secho(f, fg='red', err=True)
        raise SystemExit
    else:
        filename = files[0]
    try:
        with open(filename, encoding='utf-8') as fd:
            text = fd.read()
    except FileNotFoundError:
        click.secho('File not found!', fg='red', err=True)
        raise SystemExit
    else:
        click.echo_via_pager(text)
