import os.path
import glob

import click

from typing import Tuple, List, Union

CONFIG_PATH = click.get_app_dir('patternutils')


def load_template(template_name: str, command_name: str) -> Tuple[str, str]:
    try:
        files = find_templates(template_name, command_name)
        if len(files) == 1:
            with open(files[0], encoding='utf-8') as f:
                template_lines = f.read()
        elif len(files) == 0:
            raise FileNotFoundError
        elif len(files) > 1:
            click.secho('More then one file found!', fg='red', err=True)
            raise SystemExit
    except FileNotFoundError:
        raise
    else:
        output_pattern = template_lines.replace('\n', '')
        template_file = files[0]
        if template_file.endswith('regex'):
            template_engine = 'regex'
        elif template_file.endswith('jinja2'):
            template_engine = 'jinja2'
        else:
            template_engine = 'python'

    return output_pattern, template_engine


def find_templates(template_name: str, command_name: str=None) -> List[str]:
    template_glob = '.'.join((template_name, '*'))

    command_glob = '*' if command_name is None else command_name
    command_glob = os.path.join(CONFIG_PATH, command_glob)
    return glob.glob(os.path.join(command_glob, template_glob))
