from typing import Callable

try:
    import jinja2
except ImportError:
    jinja_feature = False
else:
    import shlex

    from datetime import datetime

    jinja_feature = True
    jinja_env = jinja2.Environment()

    def to_datetime(value: str, format: str='%Y-%m-%d') -> datetime:
        return datetime.strptime(value, format)

    def format_datetime(value: datetime, format: str='%Y-%m-%d') -> str:
        return value.strftime(format)

    def to_int(value: str, base: int=10) -> int:
        return int(value, base=base)

    def to_float(value: str) -> float:
        return float(value)

    def quote(value: str) -> str:
        return shlex.quote(value)

    jinja_env.filters['datetime'] = to_datetime
    jinja_env.filters['datetimeformat'] = format_datetime
    jinja_env.filters['int'] = to_int
    jinja_env.filters['float'] = to_float
    jinja_env.filters['quote'] = quote


def get_render_function(template: str, *, engine: str='python') -> Callable[..., str]:
    if engine == 'jinja2':
        if jinja_feature:
            env_template = jinja_env.from_string(template)
            render_function = env_template.render
        else:
            raise RuntimeError('jinja2 not installed!')
    else:
        render_function = template.format
    return render_function
